@@ -205,7 +205,6 @@ pub mod macros {
     macro_rules! vec_parse {
         ($vec:expr) => {{
             let mut v2: Vec<f64> = Vec::new();
-
             if $vec.len() > 0 {
                 match &$vec[0].parse::<f64>() {
                     Ok(r) => {
@@ -221,15 +220,119 @@ pub mod macros {
             v2
         }};
     }
-
-    pub fn run() {
-        let v1 = vec_parse!(vec!["15.", "2.9"]);
-        let v2 = vec_parse!(vec!["15", "2"]);
-        let v3 = vec_parse!(vec![".15", ".2"]);
-        assert_eq!(vec![15.0, 2.9], v1);
-        assert_eq!(vec![15.0, 2.0], v2);
-        assert_eq!(vec![0.15, 0.2], v3);
+    /// vec_as_string!() used to
+    /// convert vec item to String,
+    /// type can be Vec\<i32\> Vec\<i64\> Vec\<i128\> Vec\<f32\> Vec\<f64\>,Vec\<&str\>
+    /// return Vec\<String\>
+    /// ```rust
+    ///let v1 = vec_as_string!(vec!["15.", "2.9"]);
+    ///let v2 = vec_as_string!(vec![15, 2]);
+    ///let v3 = vec_as_string!(vec![0.15, 0.2]);
+    ///assert_eq!(vec!["15.", "2.9"], v1);
+    ///assert_eq!(vec!["15", "2"], v2);
+    ///assert_eq!(vec!["0.15", "0.2"], v3);
+    /// ```
+    #[macro_export]
+    macro_rules! vec_as_string {
+        ($vec:expr) => {{
+            let mut v2: Vec<String> = Vec::new();
+            if $vec.len() > 0 {
+                let vec1 = $vec.clone();
+                v2 = vec1
+                    .iter()
+                    .map(|x| format!("{}", &x))
+                    .collect::<Vec<String>>();
+            }
+            v2
+        }};
     }
+    /// snail_sort!() return the array elements arranged from outermost elements to the middle element, traveling clockwise.n x n
+    /// type can be Vec\<Vec\<T\>\>
+    /// return Vec\<T\>
+    /// ```rust
+    ///let v1 = snail_sort!(vec![vec![1, 2, 3], vec![4, 5, 6], vec![7, 8, 9]]);
+    ///let v2 =  snail_sort!(vec![vec![1.1, 2.1, 3.1],vec![4.1, 5.1, 6.1],vec![7.1, 8.1, 9.1]]);
+    ///assert_eq!(vec![1, 2, 3, 6, 9, 8, 7, 4, 5], v1);
+    ///assert_eq!(vec![1.1, 2.1, 3.1, 6.1, 9.1, 8.1, 7.1, 4.1, 5.1], v2);
+    /// ```
+    #[macro_export]
+    macro_rules! snail_sort {
+        ($vec:expr) => {{
+            fn snail<T: Copy>(matrix: &[Vec<T>]) -> Vec<T> {
+                let mut ret = Vec::new();
+                if matrix.len() == 0 {
+                    return ret;
+                }
+                let mut width = matrix[0].len();
+                let mut height = matrix.len();
+                let mut cycle = 0;
+                while width > 0 && height > 0 {
+                    for x in cycle..width {
+                        ret.push(matrix[cycle][x]);
+                    }
+                    for y in cycle + 1..height {
+                        ret.push(matrix[y][width - 1]);
+                    }
+                    for x in (cycle..width - 1).rev() {
+                        ret.push(matrix[height - 1][x]);
+                    }
+                    for y in (cycle + 1..height - 1).rev() {
+                        ret.push(matrix[y][cycle]);
+                    }
+                    cycle += 1;
+                    width -= 1;
+                    height -= 1;
+                }
+                ret
+            }
+            let vec = $vec.clone();
+            snail(&vec)
+        }};
+    }
+
+    /// multiply_matrix!() return the mutiply result of two matrix
+    /// take two matrix and type can be Vec\<Vec\<f64\>\>
+    /// return Vec\<f64\>
+    /// ```rust
+    ///let m1: Vec<Vec<f64>> = vec![vec![1.0, 2.0], vec![1.0, 2.0]];
+    ///let m2: Vec<Vec<f64>> = vec![vec![0.0, 0.0], vec![0.0, 0.5]];
+    ///let mul_result = multiply_matrix!(&m2, &m1);
+    ///assert_eq!(mul_result, vec![[0.0, 0.0], [0.5, 1.0]]);
+    /// ```
+    #[macro_export]
+    macro_rules! multiply_matrix {
+        ($vec1:expr,$vec2:expr) => {{
+            pub fn new(m: usize, n: usize) -> Vec<Vec<f64>> {
+                //returb 0 created n*m matrix
+                let mut mat = Vec::new();
+                for i in 0..m {
+                    mat.push(vec![0; n].iter().map(|s| *s as f64).collect::<Vec<_>>());
+                }
+                let vec = mat.clone();
+                vec
+            }
+            pub fn multiply_matrix<'a>(
+                matrix1: &'a Vec<Vec<f64>>,
+                matrix2: &'a Vec<Vec<f64>>,
+            ) -> Vec<Vec<f64>> {
+                let m = &matrix2.len();
+                let n = &matrix1[0].len();
+                let mut matrix3: Vec<Vec<f64>> = new(*&matrix2.len(), *&matrix2[0].len());
+                for i in 0..*&matrix2.len() {
+                    for j in 0..*&matrix2[0].len() {
+                        for k in 0..*m {
+                            matrix3[i][j] += matrix1[i][k] * matrix2[k][j];
+                        }
+                    }
+                }
+                matrix3
+            }
+            let vec1 = $vec1.clone();
+            let vec2 = $vec2.clone();
+            multiply_matrix(&vec1, &vec2)
+        }};
+    }
+    pub fn run() {}
 }
 
 #[cfg(test)]
